@@ -35,7 +35,7 @@ g_i_marshalling_tests_boolean_in_false (gboolean bool_)
 
 /**
  * g_i_marshalling_tests_boolean_out_true:
- * bool_: (out):
+ * @bool_: (out):
  */
 void
 g_i_marshalling_tests_boolean_out_true (gboolean *bool_)
@@ -45,7 +45,7 @@ g_i_marshalling_tests_boolean_out_true (gboolean *bool_)
 
 /**
  * g_i_marshalling_tests_boolean_out_false:
- * bool_: (out):
+ * @bool_: (out):
  */
 void
 g_i_marshalling_tests_boolean_out_false (gboolean *bool_)
@@ -55,7 +55,7 @@ g_i_marshalling_tests_boolean_out_false (gboolean *bool_)
 
 /**
  * g_i_marshalling_tests_boolean_inout_true_false:
- * bool_: (inout):
+ * @bool_: (inout):
  */
 void
 g_i_marshalling_tests_boolean_inout_true_false (gboolean *bool_)
@@ -66,7 +66,7 @@ g_i_marshalling_tests_boolean_inout_true_false (gboolean *bool_)
 
 /**
  * g_i_marshalling_tests_boolean_inout_false_true:
- * bool_: (inout):
+ * @bool_: (inout):
  */
 void
 g_i_marshalling_tests_boolean_inout_false_true (gboolean *bool_)
@@ -104,7 +104,7 @@ g_i_marshalling_tests_int8_in_min (gint8 int8)
 
 /**
  * g_i_marshalling_tests_int8_out_max:
- * int8: (out):
+ * @int8: (out):
  */
 void
 g_i_marshalling_tests_int8_out_max (gint8 *int8)
@@ -114,7 +114,7 @@ g_i_marshalling_tests_int8_out_max (gint8 *int8)
 
 /**
  * g_i_marshalling_tests_int8_out_min:
- * int8: (out):
+ * @int8: (out):
  */
 void
 g_i_marshalling_tests_int8_out_min (gint8 *int8)
@@ -124,7 +124,7 @@ g_i_marshalling_tests_int8_out_min (gint8 *int8)
 
 /**
  * g_i_marshalling_tests_int8_inout_max_min:
- * int8: (inout):
+ * @int8: (inout):
  */
 void
 g_i_marshalling_tests_int8_inout_max_min (gint8 *int8)
@@ -135,7 +135,7 @@ g_i_marshalling_tests_int8_inout_max_min (gint8 *int8)
 
 /**
  * g_i_marshalling_tests_int8_inout_min_max:
- * int8: (inout):
+ * @int8: (inout):
  */
 void
 g_i_marshalling_tests_int8_inout_min_max (gint8 *int8)
@@ -159,7 +159,7 @@ g_i_marshalling_tests_uint8_in (guint8 uint8)
 
 /**
  * g_i_marshalling_tests_uint8_out:
- * uint8: (out):
+ * @uint8: (out):
  */
 void
 g_i_marshalling_tests_uint8_out (guint8 *uint8)
@@ -169,7 +169,7 @@ g_i_marshalling_tests_uint8_out (guint8 *uint8)
 
 /**
  * g_i_marshalling_tests_uint8_inout:
- * uint8: (inout):
+ * @uint8: (inout):
  */
 void
 g_i_marshalling_tests_uint8_inout (guint8 *uint8)
@@ -205,7 +205,7 @@ g_i_marshalling_tests_int16_in_min (gint16 int16)
 
 /**
  * g_i_marshalling_tests_int16_out_max:
- * int16: (out):
+ * @int16: (out):
  */
 void
 g_i_marshalling_tests_int16_out_max (gint16 *int16)
@@ -215,7 +215,7 @@ g_i_marshalling_tests_int16_out_max (gint16 *int16)
 
 /**
  * g_i_marshalling_tests_int16_out_min:
- * int16: (out):
+ * @int16: (out):
  */
 void
 g_i_marshalling_tests_int16_out_min (gint16 *int16)
@@ -225,7 +225,7 @@ g_i_marshalling_tests_int16_out_min (gint16 *int16)
 
 /**
  * g_i_marshalling_tests_int16_inout_max_min:
- * int16: (inout):
+ * @int16: (inout):
  */
 void
 g_i_marshalling_tests_int16_inout_max_min (gint16 *int16)
@@ -236,7 +236,7 @@ g_i_marshalling_tests_int16_inout_max_min (gint16 *int16)
 
 /**
  * g_i_marshalling_tests_int16_inout_min_max:
- * int16: (inout):
+ * @int16: (inout):
  */
 void
 g_i_marshalling_tests_int16_inout_min_max (gint16 *int16)
@@ -260,7 +260,7 @@ g_i_marshalling_tests_uint16_in (guint16 uint16)
 
 /**
  * g_i_marshalling_tests_uint16_out:
- * uint16: (out):
+ * @uint16: (out):
  */
 void
 g_i_marshalling_tests_uint16_out (guint16 *uint16)
@@ -270,7 +270,7 @@ g_i_marshalling_tests_uint16_out (guint16 *uint16)
 
 /**
  * g_i_marshalling_tests_uint16_inout:
- * uint16: (inout):
+ * @uint16: (inout):
  */
 void
 g_i_marshalling_tests_uint16_inout (guint16 *uint16)
@@ -306,7 +306,7 @@ g_i_marshalling_tests_int32_in_min (gint32 int32)
 
 /**
  * g_i_marshalling_tests_int32_out_max:
- * int32: (out):
+ * @int32: (out):
  */
 void
 g_i_marshalling_tests_int32_out_max (gint32 *int32)
@@ -316,7 +316,7 @@ g_i_marshalling_tests_int32_out_max (gint32 *int32)
 
 /**
  * g_i_marshalling_tests_int32_out_min:
- * int32: (out):
+ * @int32: (out):
  */
 void
 g_i_marshalling_tests_int32_out_min (gint32 *int32)
@@ -326,7 +326,7 @@ g_i_marshalling_tests_int32_out_min (gint32 *int32)
 
 /**
  * g_i_marshalling_tests_int32_inout_max_min:
- * int32: (inout):
+ * @int32: (inout):
  */
 void
 g_i_marshalling_tests_int32_inout_max_min (gint32 *int32)
@@ -337,7 +337,7 @@ g_i_marshalling_tests_int32_inout_max_min (gint32 *int32)
 
 /**
  * g_i_marshalling_tests_int32_inout_min_max:
- * int32: (inout):
+ * @int32: (inout):
  */
 void
 g_i_marshalling_tests_int32_inout_min_max (gint32 *int32)
@@ -361,7 +361,7 @@ g_i_marshalling_tests_uint32_in (guint32 uint32)
 
 /**
  * g_i_marshalling_tests_uint32_out:
- * uint32: (out):
+ * @uint32: (out):
  */
 void
 g_i_marshalling_tests_uint32_out (guint32 *uint32)
@@ -371,7 +371,7 @@ g_i_marshalling_tests_uint32_out (guint32 *uint32)
 
 /**
  * g_i_marshalling_tests_uint32_inout:
- * uint32: (inout):
+ * @uint32: (inout):
  */
 void
 g_i_marshalling_tests_uint32_inout (guint32 *uint32)
@@ -407,7 +407,7 @@ g_i_marshalling_tests_int64_in_min (gint64 int64)
 
 /**
  * g_i_marshalling_tests_int64_out_max:
- * int64: (out):
+ * @int64: (out):
  */
 void
 g_i_marshalling_tests_int64_out_max (gint64 *int64)
@@ -417,7 +417,7 @@ g_i_marshalling_tests_int64_out_max (gint64 *int64)
 
 /**
  * g_i_marshalling_tests_int64_out_min:
- * int64: (out):
+ * @int64: (out):
  */
 void
 g_i_marshalling_tests_int64_out_min (gint64 *int64)
@@ -427,7 +427,7 @@ g_i_marshalling_tests_int64_out_min (gint64 *int64)
 
 /**
  * g_i_marshalling_tests_int64_inout_max_min:
- * int64: (inout):
+ * @int64: (inout):
  */
 void
 g_i_marshalling_tests_int64_inout_max_min (gint64 *int64)
@@ -438,7 +438,7 @@ g_i_marshalling_tests_int64_inout_max_min (gint64 *int64)
 
 /**
  * g_i_marshalling_tests_int64_inout_min_max:
- * int64: (inout):
+ * @int64: (inout):
  */
 void
 g_i_marshalling_tests_int64_inout_min_max (gint64 *int64)
@@ -462,7 +462,7 @@ g_i_marshalling_tests_uint64_in (guint64 uint64)
 
 /**
  * g_i_marshalling_tests_uint64_out:
- * uint64: (out):
+ * @uint64: (out):
  */
 void
 g_i_marshalling_tests_uint64_out (guint64 *uint64)
@@ -472,7 +472,7 @@ g_i_marshalling_tests_uint64_out (guint64 *uint64)
 
 /**
  * g_i_marshalling_tests_uint64_inout:
- * uint64: (inout):
+ * @uint64: (inout):
  */
 void
 g_i_marshalling_tests_uint64_inout (guint64 *uint64)
@@ -508,7 +508,7 @@ g_i_marshalling_tests_short_in_min (gshort short_)
 
 /**
  * g_i_marshalling_tests_short_out_max:
- * short_: (out):
+ * @short_: (out):
  */
 void
 g_i_marshalling_tests_short_out_max (gshort *short_)
@@ -518,7 +518,7 @@ g_i_marshalling_tests_short_out_max (gshort *short_)
 
 /**
  * g_i_marshalling_tests_short_out_min:
- * short_: (out):
+ * @short_: (out):
  */
 void
 g_i_marshalling_tests_short_out_min (gshort *short_)
@@ -528,7 +528,7 @@ g_i_marshalling_tests_short_out_min (gshort *short_)
 
 /**
  * g_i_marshalling_tests_short_inout_max_min:
- * short_: (inout):
+ * @short_: (inout):
  */
 void
 g_i_marshalling_tests_short_inout_max_min (gshort *short_)
@@ -539,7 +539,7 @@ g_i_marshalling_tests_short_inout_max_min (gshort *short_)
 
 /**
  * g_i_marshalling_tests_short_inout_min_max:
- * short_: (inout):
+ * @short_: (inout):
  */
 void
 g_i_marshalling_tests_short_inout_min_max (gshort *short_)
@@ -563,7 +563,7 @@ g_i_marshalling_tests_ushort_in (gushort ushort)
 
 /**
  * g_i_marshalling_tests_ushort_out:
- * ushort: (out):
+ * @ushort: (out):
  */
 void
 g_i_marshalling_tests_ushort_out (gushort *ushort)
@@ -573,7 +573,7 @@ g_i_marshalling_tests_ushort_out (gushort *ushort)
 
 /**
  * g_i_marshalling_tests_ushort_inout:
- * ushort: (inout):
+ * @ushort: (inout):
  */
 void
 g_i_marshalling_tests_ushort_inout (gushort *ushort)
@@ -609,7 +609,7 @@ g_i_marshalling_tests_int_in_min (gint int_)
 
 /**
  * g_i_marshalling_tests_int_out_max:
- * int_: (out):
+ * @int_: (out):
  */
 void
 g_i_marshalling_tests_int_out_max (gint *int_)
@@ -619,7 +619,7 @@ g_i_marshalling_tests_int_out_max (gint *int_)
 
 /**
  * g_i_marshalling_tests_int_out_min:
- * int_: (out):
+ * @int_: (out):
  */
 void
 g_i_marshalling_tests_int_out_min (gint *int_)
@@ -629,7 +629,7 @@ g_i_marshalling_tests_int_out_min (gint *int_)
 
 /**
  * g_i_marshalling_tests_int_inout_max_min:
- * int_: (inout):
+ * @int_: (inout):
  */
 void
 g_i_marshalling_tests_int_inout_max_min (gint *int_)
@@ -640,7 +640,7 @@ g_i_marshalling_tests_int_inout_max_min (gint *int_)
 
 /**
  * g_i_marshalling_tests_int_inout_min_max:
- * int_: (inout):
+ * @int_: (inout):
  */
 void
 g_i_marshalling_tests_int_inout_min_max (gint *int_)
@@ -664,7 +664,7 @@ g_i_marshalling_tests_uint_in (guint uint)
 
 /**
  * g_i_marshalling_tests_uint_out:
- * uint: (out):
+ * @uint: (out):
  */
 void
 g_i_marshalling_tests_uint_out (guint *uint)
@@ -674,7 +674,7 @@ g_i_marshalling_tests_uint_out (guint *uint)
 
 /**
  * g_i_marshalling_tests_uint_inout:
- * uint: (inout):
+ * @uint: (inout):
  */
 void
 g_i_marshalling_tests_uint_inout (guint *uint)
@@ -710,7 +710,7 @@ g_i_marshalling_tests_long_in_min (glong long_)
 
 /**
  * g_i_marshalling_tests_long_out_max:
- * long_: (out):
+ * @long_: (out):
  */
 void
 g_i_marshalling_tests_long_out_max (glong *long_)
@@ -720,7 +720,7 @@ g_i_marshalling_tests_long_out_max (glong *long_)
 
 /**
  * g_i_marshalling_tests_long_out_min:
- * long_: (out):
+ * @long_: (out):
  */
 void
 g_i_marshalling_tests_long_out_min (glong *long_)
@@ -730,7 +730,7 @@ g_i_marshalling_tests_long_out_min (glong *long_)
 
 /**
  * g_i_marshalling_tests_long_inout_max_min:
- * long_: (inout):
+ * @long_: (inout):
  */
 void
 g_i_marshalling_tests_long_inout_max_min (glong *long_)
@@ -741,7 +741,7 @@ g_i_marshalling_tests_long_inout_max_min (glong *long_)
 
 /**
  * g_i_marshalling_tests_long_inout_min_max:
- * long_: (inout):
+ * @long_: (inout):
  */
 void
 g_i_marshalling_tests_long_inout_min_max (glong *long_)
@@ -765,7 +765,7 @@ g_i_marshalling_tests_ulong_in (gulong ulong)
 
 /**
  * g_i_marshalling_tests_ulong_out:
- * ulong: (out):
+ * @ulong: (out):
  */
 void
 g_i_marshalling_tests_ulong_out (gulong *ulong)
@@ -775,7 +775,7 @@ g_i_marshalling_tests_ulong_out (gulong *ulong)
 
 /**
  * g_i_marshalling_tests_ulong_inout:
- * ulong: (inout):
+ * @ulong: (inout):
  */
 void
 g_i_marshalling_tests_ulong_inout (gulong *ulong)
@@ -811,7 +811,7 @@ g_i_marshalling_tests_ssize_in_min (gssize ssize)
 
 /**
  * g_i_marshalling_tests_ssize_out_max:
- * ssize: (out):
+ * @ssize: (out):
  */
 void
 g_i_marshalling_tests_ssize_out_max (gssize *ssize)
@@ -821,7 +821,7 @@ g_i_marshalling_tests_ssize_out_max (gssize *ssize)
 
 /**
  * g_i_marshalling_tests_ssize_out_min:
- * ssize: (out):
+ * @ssize: (out):
  */
 void
 g_i_marshalling_tests_ssize_out_min (gssize *ssize)
@@ -831,7 +831,7 @@ g_i_marshalling_tests_ssize_out_min (gssize *ssize)
 
 /**
  * g_i_marshalling_tests_ssize_inout_max_min:
- * ssize: (inout):
+ * @ssize: (inout):
  */
 void
 g_i_marshalling_tests_ssize_inout_max_min (gssize *ssize)
@@ -842,7 +842,7 @@ g_i_marshalling_tests_ssize_inout_max_min (gssize *ssize)
 
 /**
  * g_i_marshalling_tests_ssize_inout_min_max:
- * ssize: (inout):
+ * @ssize: (inout):
  */
 void
 g_i_marshalling_tests_ssize_inout_min_max (gssize *ssize)
@@ -866,7 +866,7 @@ g_i_marshalling_tests_size_in (gsize size)
 
 /**
  * g_i_marshalling_tests_size_out:
- * size: (out):
+ * @size: (out):
  */
 void
 g_i_marshalling_tests_size_out (gsize *size)
@@ -876,7 +876,7 @@ g_i_marshalling_tests_size_out (gsize *size)
 
 /**
  * g_i_marshalling_tests_size_inout:
- * size: (inout):
+ * @size: (inout):
  */
 void
 g_i_marshalling_tests_size_inout (gsize *size)
@@ -900,7 +900,7 @@ g_i_marshalling_tests_float_in (gfloat float_)
 
 /**
  * g_i_marshalling_tests_float_out:
- * float_: (out):
+ * @float_: (out):
  */
 void
 g_i_marshalling_tests_float_out (gfloat *float_)
@@ -910,7 +910,7 @@ g_i_marshalling_tests_float_out (gfloat *float_)
 
 /**
  * g_i_marshalling_tests_float_inout:
- * float_: (inout):
+ * @float_: (inout):
  */
 void
 g_i_marshalling_tests_float_inout (gfloat *float_)
@@ -934,7 +934,7 @@ g_i_marshalling_tests_double_in (gdouble double_)
 
 /**
  * g_i_marshalling_tests_double_out:
- * double_: (out):
+ * @double_: (out):
  */
 void
 g_i_marshalling_tests_double_out (gdouble *double_)
@@ -944,7 +944,7 @@ g_i_marshalling_tests_double_out (gdouble *double_)
 
 /**
  * g_i_marshalling_tests_double_inout:
- * double_: (inout):
+ * @double_: (inout):
  */
 void
 g_i_marshalling_tests_double_inout (gdouble *double_)
@@ -968,7 +968,7 @@ g_i_marshalling_tests_time_t_in (time_t time_t_)
 
 /**
  * g_i_marshalling_tests_time_t_out:
- * time_t_: (out):
+ * @time_t_: (out):
  */
 void
 g_i_marshalling_tests_time_t_out (time_t *time_t_)
@@ -978,7 +978,7 @@ g_i_marshalling_tests_time_t_out (time_t *time_t_)
 
 /**
  * g_i_marshalling_tests_time_t_inout:
- * time_t_: (inout):
+ * @time_t_: (inout):
  */
 void
 g_i_marshalling_tests_time_t_inout (time_t *time_t_)
@@ -1002,7 +1002,7 @@ g_i_marshalling_tests_gtype_in (GType gtype)
 
 /**
  * g_i_marshalling_tests_gtype_out:
- * gtype: (out):
+ * @gtype: (out):
  */
 void
 g_i_marshalling_tests_gtype_out (GType *gtype)
@@ -1012,7 +1012,7 @@ g_i_marshalling_tests_gtype_out (GType *gtype)
 
 /**
  * g_i_marshalling_tests_gtype_inout:
- * gtype: (inout):
+ * @gtype: (inout):
  */
 void
 g_i_marshalling_tests_gtype_inout (GType *gtype)
@@ -1049,7 +1049,7 @@ g_i_marshalling_tests_utf8_full_in (gchar *utf8)
 
 /**
  * g_i_marshalling_tests_utf8_none_out:
- * utf8: (out) (transfer none):
+ * @utf8: (out) (transfer none):
  */
 void
 g_i_marshalling_tests_utf8_none_out (gchar **utf8)
@@ -1059,7 +1059,7 @@ g_i_marshalling_tests_utf8_none_out (gchar **utf8)
 
 /**
  * g_i_marshalling_tests_utf8_full_out:
- * utf8: (out) (transfer full):
+ * @utf8: (out) (transfer full):
  */
 void
 g_i_marshalling_tests_utf8_full_out (gchar **utf8)
@@ -1067,9 +1067,23 @@ g_i_marshalling_tests_utf8_full_out (gchar **utf8)
     *utf8 = g_strdup(G_I_MARSHALLING_TESTS_CONSTANT_UTF8);
 }
 
+/**
+ * g_i_marshalling_tests_utf8_dangling_out:
+ * @utf8: (out) (transfer full):
+ */
+void
+g_i_marshalling_tests_utf8_dangling_out (gchar **utf8)
+{
+    /* Intentionally don't touch the pointer to see how
+       the bindings handle this case.  Bindings should be
+       robust against broken C functions and can initialize 
+       even OUT vlues to NULL.
+    */
+}
+
 /**
  * g_i_marshalling_tests_utf8_none_inout:
- * utf8: (inout) (transfer none):
+ * @utf8: (inout) (transfer none):
  */
 void
 g_i_marshalling_tests_utf8_none_inout (gchar **utf8)
@@ -1080,7 +1094,7 @@ g_i_marshalling_tests_utf8_none_inout (gchar **utf8)
 
 /**
  * g_i_marshalling_tests_utf8_full_inout:
- * utf8: (inout) (transfer full):
+ * @utf8: (inout) (transfer full):
  */
 void
 g_i_marshalling_tests_utf8_full_inout (gchar **utf8)
@@ -1303,6 +1317,361 @@ g_i_marshalling_tests_array_zero_terminated_inout (gchar ***utf8s)
 }
 
 
+/**
+ * g_i_marshalling_tests_garray_int_none_return:
+ * Returns: (element-type gint) (transfer none):
+ */
+GArray *
+g_i_marshalling_tests_garray_int_none_return (void)
+{
+    static GArray *array = NULL;
+    gint i;
+
+    if (array == NULL) {
+        array = g_array_new (TRUE, TRUE, sizeof (gint));
+        for (i = -1; i < 3; i++)
+            g_array_append_val (array, i);
+    }
+
+    return array;
+}
+
+/**
+ * g_i_marshalling_tests_garray_utf8_none_return:
+ * Returns: (element-type utf8) (transfer none):
+ */
+GArray *
+g_i_marshalling_tests_garray_utf8_none_return (void)
+{
+    static GArray *array = NULL;
+    static gchar *values[] = {"0", "1", "2", NULL};
+    gint i;
+
+    if (array == NULL) {
+        array = g_array_new (TRUE, TRUE, sizeof (gchar *));
+        for (i = 0; values[i]; i++)
+            g_array_append_val (array, values[i]);
+    }
+
+    return array;
+}
+
+/**
+ * g_i_marshalling_tests_garray_utf8_container_return:
+ * Returns: (element-type utf8) (transfer container):
+ */
+GArray *
+g_i_marshalling_tests_garray_utf8_container_return (void)
+{
+    GArray *array = NULL;
+    static gchar *values[] = {"0", "1", "2", NULL};
+    gint i;
+
+    array = g_array_new (TRUE, TRUE, sizeof (gchar *));
+    for (i = 0; values[i]; i++)
+        g_array_append_val (array, values[i]);
+
+    return array;
+}
+
+/**
+ * g_i_marshalling_tests_garray_utf8_full_return:
+ * Returns: (element-type utf8) (transfer full):
+ */
+GArray *
+g_i_marshalling_tests_garray_utf8_full_return (void)
+{
+    GArray *array = NULL;
+    static gchar *values[] = {"0", "1", "2", NULL};
+    gint i;
+
+    array = g_array_new (TRUE, TRUE, sizeof (gchar *));
+    for (i = 0; values[i]; i++) {
+        gchar *str = g_strdup (values[i]);
+        g_array_append_val (array, str);
+    }
+
+    return array;
+}
+
+/**
+ * g_i_marshalling_tests_garray_int_none_in:
+ * @array_: (element-type gint) (transfer none):
+ */
+void
+g_i_marshalling_tests_garray_int_none_in (GArray *array_)
+{
+    g_assert (array_->len == 4);
+    g_assert (g_array_index (array_, gint, 0) == -1);
+    g_assert (g_array_index (array_, gint, 1) == 0);
+    g_assert (g_array_index (array_, gint, 2) == 1);
+    g_assert (g_array_index (array_, gint, 3) == 2);
+}
+
+/**
+ * g_i_marshalling_tests_garray_utf8_none_in:
+ * @array_: (element-type utf8) (transfer none):
+ */
+void
+g_i_marshalling_tests_garray_utf8_none_in (GArray *array_)
+{
+    g_assert (array_->len == 3);
+    g_assert (strcmp (g_array_index (array_, gchar*, 0), "0") == 0);
+    g_assert (strcmp (g_array_index (array_, gchar*, 1), "1") == 0);
+    g_assert (strcmp (g_array_index (array_, gchar*, 2), "2") == 0);
+}
+
+/**
+ * g_i_marshalling_tests_garray_utf8_container_in:
+ * @array_: (element-type utf8) (transfer container):
+ */
+void
+g_i_marshalling_tests_garray_utf8_container_in (GArray *array_)
+{
+    g_assert (array_->len == 3);
+    g_assert (strcmp (g_array_index (array_, gchar*, 0), "0") == 0);
+    g_assert (strcmp (g_array_index (array_, gchar*, 1), "1") == 0);
+    g_assert (strcmp (g_array_index (array_, gchar*, 2), "2") == 0);
+    g_array_free (array_, TRUE);
+}
+
+/**
+ * g_i_marshalling_tests_garray_utf8_full_in:
+ * @array_: (element-type utf8) (transfer full):
+ */
+void
+g_i_marshalling_tests_garray_utf8_full_in (GArray *array_)
+{
+    g_assert (array_->len == 3);
+    g_assert (strcmp (g_array_index (array_, gchar*, 0), "0") == 0);
+    g_assert (strcmp (g_array_index (array_, gchar*, 1), "1") == 0);
+    g_assert (strcmp (g_array_index (array_, gchar*, 2), "2") == 0);
+    g_free (g_array_index (array_, gchar*, 0));
+    g_free (g_array_index (array_, gchar*, 1));
+    g_free (g_array_index (array_, gchar*, 2));
+    g_array_free (array_, TRUE);
+}
+
+/**
+ * g_i_marshalling_tests_garray_utf8_none_out:
+ * @array_: (out) (element-type utf8) (transfer none):
+ */
+void
+g_i_marshalling_tests_garray_utf8_none_out (GArray **array_)
+{
+    static GArray *internal = NULL;
+    static gchar *values[] = {"0", "1", "2", NULL};
+    gint i;
+
+    if (internal == NULL) {
+        internal = g_array_new (TRUE, TRUE, sizeof (gchar *));
+        for (i = 0; values[i]; i++)
+            g_array_append_val (internal, values[i]);
+    }
+
+    *array_ = internal;
+}
+
+/**
+ * g_i_marshalling_tests_garray_utf8_container_out:
+ * @array_: (out) (element-type utf8) (transfer container):
+ */
+void
+g_i_marshalling_tests_garray_utf8_container_out (GArray **array_)
+{
+    static gchar *values[] = {"0", "1", "2", NULL};
+    gint i;
+
+    *array_ = NULL;
+
+    *array_ = g_array_new (TRUE, TRUE, sizeof (gchar *));
+    for (i = 0; values[i]; i++)
+        g_array_append_val (*array_, values[i]);
+}
+
+/**
+ * g_i_marshalling_tests_garray_utf8_full_out:
+ * @array_: (out) (element-type utf8) (transfer full):
+ */
+void
+g_i_marshalling_tests_garray_utf8_full_out (GArray **array_)
+{
+    static gchar *values[] = {"0", "1", "2", NULL};
+    gint i;
+
+    *array_ = NULL;
+
+    *array_ = g_array_new (TRUE, TRUE, sizeof (gchar *));
+    for (i = 0; values[i]; i++) {
+        gchar *str = g_strdup (values[i]);
+        g_array_append_val (*array_, str);
+    }
+}
+
+/**
+ * g_i_marshalling_tests_garray_utf8_none_inout:
+ * @array_: (inout) (element-type utf8) (transfer none):
+ */
+void
+g_i_marshalling_tests_garray_utf8_none_inout (GArray **array_)
+{
+    static GArray *internal = NULL;
+    static gchar *values[] = {"-2", "-1", "0", "1", NULL};
+    gint i;
+
+    g_assert ((*array_)->len == 3);
+    g_assert (strcmp (g_array_index (*array_, gchar*, 0), "0") == 0);
+    g_assert (strcmp (g_array_index (*array_, gchar*, 1), "1") == 0);
+    g_assert (strcmp (g_array_index (*array_, gchar*, 2), "2") == 0);
+
+    if (internal == NULL) {
+        internal = g_array_new (TRUE, TRUE, sizeof (gchar *));
+        for (i = 0; values[i]; i++)
+            g_array_append_val (internal, values[i]);
+    }
+
+    *array_ = internal;
+}
+
+/**
+ * g_i_marshalling_tests_garray_utf8_container_inout:
+ * @array_: (inout) (element-type utf8) (transfer container):
+ */
+void
+g_i_marshalling_tests_garray_utf8_container_inout (GArray **array_)
+{
+    static gchar *val1 = "-1";
+    static gchar *val2 = "-2";
+
+    g_assert ((*array_)->len == 3);
+    g_assert (strcmp (g_array_index (*array_, gchar*, 0), "0") == 0);
+    g_assert (strcmp (g_array_index (*array_, gchar*, 1), "1") == 0);
+    g_assert (strcmp (g_array_index (*array_, gchar*, 2), "2") == 0);
+
+    g_array_remove_index (*array_, (*array_)->len - 1);
+
+    g_array_prepend_val (*array_, val1);
+    g_array_prepend_val (*array_, val2);
+}
+
+/**
+ * g_i_marshalling_tests_garray_utf8_full_inout:
+ * @array_: (inout) (element-type utf8) (transfer full):
+ */
+void
+g_i_marshalling_tests_garray_utf8_full_inout (GArray **array_)
+{
+    static gchar *val1 = "-1";
+    static gchar *val2 = "-2";
+    gchar *val;
+
+    g_assert ((*array_)->len == 3);
+    g_assert (strcmp (g_array_index (*array_, gchar*, 0), "0") == 0);
+    g_assert (strcmp (g_array_index (*array_, gchar*, 1), "1") == 0);
+    g_assert (strcmp (g_array_index (*array_, gchar*, 2), "2") == 0);
+
+    g_free (g_array_index (*array_, gchar*, (*array_)->len - 1));
+    g_array_remove_index (*array_, (*array_)->len - 1);
+
+    val = g_strdup (val1);
+    g_array_prepend_val (*array_, val);
+
+    val = g_strdup (val2);
+    g_array_prepend_val (*array_, val);
+}
+
+/**
+ * g_i_marshalling_tests_bytearray_full_return:
+ * Returns: (transfer full):
+ */
+GByteArray *
+g_i_marshalling_tests_bytearray_full_return (void)
+{
+    GByteArray *array = NULL;
+    static gchar *values[] = {"0", "1", "2", "3", NULL};
+    gint i;
+
+    array = g_byte_array_new ();
+    for (i = 0; values[i]; i++) {
+        gchar *str = g_strdup (values[i]);
+        g_byte_array_append (array, (const guint8*)str, 1);
+    }
+
+    return array;
+
+}
+
+/**
+ * g_i_marshalling_tests_bytearray_none_in:
+ * @array_: (element-type gint) (transfer none):
+ */
+void
+g_i_marshalling_tests_bytearray_none_in (GByteArray *array_)
+{
+    g_assert (array_->len == 4);
+    g_assert (g_array_index (array_, char, 0) == '0');
+    g_assert (g_array_index (array_, char, 1) == '1');
+    g_assert (g_array_index (array_, char, 2) == '2');
+    g_assert (g_array_index (array_, char, 3) == '3');
+}
+
+/**
+ * g_i_marshalling_tests_gstrv_return:
+ *
+ * Returns: an array of strings
+ */
+GStrv
+g_i_marshalling_tests_gstrv_return (void)
+{
+    GStrv values = g_new0 (gchar*, 4);
+    values[0] = g_strdup ("0");
+    values[1] = g_strdup ("1");
+    values[2] = g_strdup ("2");
+    values[3] = NULL;
+    return values;
+}
+
+/**
+ * g_i_marshalling_tests_gstrv_in:
+ * @g_strv:
+ */
+void
+g_i_marshalling_tests_gstrv_in (GStrv g_strv)
+{
+    g_assert(g_strv_length(g_strv) == 3);
+    g_assert(strcmp(g_strv[0], "0") == 0);
+    g_assert(strcmp(g_strv[1], "1") == 0);
+    g_assert(strcmp(g_strv[2], "2") == 0);
+}
+
+/**
+ * g_i_marshalling_tests_gstrv_out:
+ * @g_strv: (out) (transfer none):
+ */
+void
+g_i_marshalling_tests_gstrv_out (GStrv *g_strv)
+{
+    static gchar *values[] = {"0", "1", "2", NULL};
+    *g_strv = values;
+}
+
+/**
+ * g_i_marshalling_tests_gstrv_inout:
+ * @g_strv: (inout) (transfer none):
+ */
+void
+g_i_marshalling_tests_gstrv_inout (GStrv *g_strv)
+{
+    static gchar *values[] = {"-1", "0", "1", "2", NULL};
+
+    g_assert(g_strv_length(*g_strv) == 3);
+    g_assert(strcmp((*g_strv)[0], "0") == 0);
+    g_assert(strcmp((*g_strv)[1], "1") == 0);
+    g_assert(strcmp((*g_strv)[2], "2") == 0);
+
+    *g_strv = values;
+}
+
 /**
  * g_i_marshalling_tests_glist_int_none_return:
  * Returns: (element-type gint) (transfer none):
@@ -2124,22 +2493,56 @@ g_i_marshalling_tests_pointer_in_return (gpointer pointer)
 }
 
 GType
-g_i_marshalling_tests_enum_get_type (void)
+g_i_marshalling_tests_genum_get_type (void)
 {
     static GType type = 0;
     if (G_UNLIKELY(type == 0)) {
         static const GEnumValue values[] = {
-            { G_I_MARSHALLING_TESTS_ENUM_VALUE1, "G_I_MARSHALLING_TESTS_ENUM_VALUE1", "value1" },
-            { G_I_MARSHALLING_TESTS_ENUM_VALUE2, "G_I_MARSHALLING_TESTS_ENUM_VALUE2", "value2" },
-            { G_I_MARSHALLING_TESTS_ENUM_VALUE3, "G_I_MARSHALLING_TESTS_ENUM_VALUE3", "value3" },
+            { G_I_MARSHALLING_TESTS_GENUM_VALUE1, "G_I_MARSHALLING_TESTS_GENUM_VALUE1", "value1" },
+            { G_I_MARSHALLING_TESTS_GENUM_VALUE2, "G_I_MARSHALLING_TESTS_GENUM_VALUE2", "value2" },
+            { G_I_MARSHALLING_TESTS_GENUM_VALUE3, "G_I_MARSHALLING_TESTS_GENUM_VALUE3", "value3" },
             { 0, NULL, NULL }
         };
-        type = g_enum_register_static (g_intern_static_string ("GIMarshallingTestsEnum"), values);
+        type = g_enum_register_static (g_intern_static_string ("GIMarshallingTestsGEnum"), values);
     }
 
     return type;
 }
 
+GIMarshallingTestsEnum
+g_i_marshalling_tests_genum_return (void)
+{
+    return G_I_MARSHALLING_TESTS_GENUM_VALUE3;
+}
+
+void
+g_i_marshalling_tests_genum_in (GIMarshallingTestsGEnum enum_)
+{
+    g_assert(enum_ == G_I_MARSHALLING_TESTS_GENUM_VALUE3);
+}
+
+/**
+ * g_i_marshalling_tests_genum_out:
+ * @enum_: (out):
+ */
+void
+g_i_marshalling_tests_genum_out (GIMarshallingTestsGEnum *enum_)
+{
+    *enum_ = G_I_MARSHALLING_TESTS_GENUM_VALUE3;
+}
+
+/**
+ * g_i_marshalling_tests_genum_inout:
+ * @enum_: (inout):
+ */
+void
+g_i_marshalling_tests_genum_inout (GIMarshallingTestsGEnum *enum_)
+{
+    g_assert(*enum_ == G_I_MARSHALLING_TESTS_GENUM_VALUE3);
+    *enum_ = G_I_MARSHALLING_TESTS_GENUM_VALUE1;
+}
+
+
 GIMarshallingTestsEnum
 g_i_marshalling_tests_enum_return (void)
 {
@@ -2424,6 +2827,11 @@ g_i_marshalling_tests__boxed_struct_return (void)
         struct_ = g_new(GIMarshallingTestsBoxedStruct, 1);
 
         struct_->long_ = 42;
+        struct_->g_strv = g_new0(gchar*, 4);
+        struct_->g_strv[0] = g_strdup("0");
+        struct_->g_strv[1] = g_strdup("1");
+        struct_->g_strv[2] = g_strdup("2");
+        struct_->g_strv[3] = NULL;
     }
 
     return struct_;
@@ -2469,6 +2877,103 @@ g_i_marshalling_tests__boxed_struct_inout (GIMarshallingTestsBoxedStruct **struc
     (*struct_)->long_ = 0;
 }
 
+static GIMarshallingTestsUnion *
+g_i_marshalling_tests_union_copy (GIMarshallingTestsUnion *union_)
+{
+    GIMarshallingTestsUnion *new_union;
+
+    new_union = g_slice_new (GIMarshallingTestsUnion);
+
+    *new_union = *union_;
+
+    return new_union;
+}
+
+static void
+g_i_marshalling_tests_union_free (GIMarshallingTestsUnion *union_)
+{
+    g_slice_free (GIMarshallingTestsUnion, union_);
+}
+
+GType
+g_i_marshalling_tests_union_get_type (void)
+{
+    static GType type = 0;
+
+    if (type == 0) {
+        type = g_boxed_type_register_static ("GIMarshallingTestsUnion",
+                (GBoxedCopyFunc) g_i_marshalling_tests_union_copy,
+                (GBoxedFreeFunc) g_i_marshalling_tests_union_free);
+    }
+
+    return type;
+}
+
+/**
+ * g_i_marshalling_tests__union_return:
+ * Returns: (transfer none):
+ */
+GIMarshallingTestsUnion *
+g_i_marshalling_tests__union_return (void)
+{
+    static GIMarshallingTestsUnion *union_ = NULL;
+
+    if (union_ == NULL) {
+        union_ = g_new(GIMarshallingTestsUnion, 1);
+
+        union_->long_ = 42;
+    }
+
+    return union_;
+}
+
+/**
+ * g_i_marshalling_tests__union_in:
+ * @union_: (transfer none):
+ */
+void
+g_i_marshalling_tests__union_in (GIMarshallingTestsUnion *union_)
+{
+    g_assert(union_->long_ == 42);
+}
+
+/**
+ * g_i_marshalling_tests__union_out:
+ * @union_: (out) (transfer none):
+ */
+void
+g_i_marshalling_tests__union_out (GIMarshallingTestsUnion **union_)
+{
+    static GIMarshallingTestsUnion *new_union = NULL;
+
+    if (new_union == NULL) {
+        new_union = g_new(GIMarshallingTestsUnion, 1);
+
+        new_union->long_ = 42;
+    }
+
+    *union_ = new_union;
+}
+
+/**
+ * g_i_marshalling_tests__union_inout:
+ * @union_: (inout) (transfer none):
+ */
+void
+g_i_marshalling_tests__union_inout (GIMarshallingTestsUnion **union_)
+{
+    g_assert((*union_)->long_ == 42);
+
+    (*union_)->long_ = 0;
+}
+
+void
+g_i_marshalling_tests_union_method (GIMarshallingTestsUnion *union_)
+{
+    g_assert(union_->long_ == 42);
+}
+
+
 
 enum
 {
@@ -2476,6 +2981,9 @@ enum
 	PROP_INT_
 };
 
+static void g_i_marshalling_tests_object_real_method_with_default_implementation (
+        GIMarshallingTestsObject *self, gint8 in);
+
 G_DEFINE_TYPE (GIMarshallingTestsObject, g_i_marshalling_tests_object, G_TYPE_OBJECT);
 
 static void
@@ -2534,6 +3042,8 @@ g_i_marshalling_tests_object_class_init (GIMarshallingTestsObjectClass *klass)
 	g_object_class_install_property (object_class, PROP_INT_,
          g_param_spec_int ("int", "Integer", "An integer", G_MININT, G_MAXINT, 0,
               G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
+
+    klass->method_with_default_implementation = g_i_marshalling_tests_object_real_method_with_default_implementation;
 }
 
 
@@ -2632,6 +3142,35 @@ g_i_marshalling_tests_object_method_int8_in (GIMarshallingTestsObject *self, gin
   G_I_MARSHALLING_TESTS_OBJECT_GET_CLASS (self)->method_int8_in (self, in);
 }
 
+/**
+ * g_i_marshalling_tests_object_method_int8_out:
+ * @out: (out):
+ */
+void
+g_i_marshalling_tests_object_method_int8_out (GIMarshallingTestsObject *self, gint8 *out)
+{
+  G_I_MARSHALLING_TESTS_OBJECT_GET_CLASS (self)->method_int8_out (self, out);
+}
+
+/**
+ * g_i_marshalling_tests_object_method_with_default_implementation:
+ * @in: (in):
+ */
+void
+g_i_marshalling_tests_object_method_with_default_implementation (GIMarshallingTestsObject *self, gint8 in)
+{
+    G_I_MARSHALLING_TESTS_OBJECT_GET_CLASS (self)->method_with_default_implementation (self, in);
+}
+
+static void
+g_i_marshalling_tests_object_real_method_with_default_implementation (GIMarshallingTestsObject *self, gint8 in)
+{
+    GValue val = {0, };
+    g_value_init (&val, G_TYPE_INT);
+    g_value_set_int (&val, in);
+    g_object_set_property (G_OBJECT (self), "int", &val);
+}
+
 
 /**
  * g_i_marshalling_tests__object_none_return:
@@ -2759,6 +3298,16 @@ g_i_marshalling_tests__object_int8_in (GIMarshallingTestsObject *object, gint8 i
   g_i_marshalling_tests_object_method_int8_in (object, in);
 }
 
+/**
+ * g_i_marshalling_tests__object_test_int8_out:
+ * @out: (out):
+ */
+void
+g_i_marshalling_tests__object_int8_out (GIMarshallingTestsObject *object, gint8 *out)
+{
+  g_i_marshalling_tests_object_method_int8_out (object, out);
+}
+
 
 G_DEFINE_TYPE (GIMarshallingTestsSubObject, g_i_marshalling_tests_sub_object, G_I_MARSHALLING_TESTS_TYPE_OBJECT);
 
@@ -2836,8 +3385,8 @@ g_i_marshalling_tests_test_interface_test_int8_in (GIMarshallingTestsInterface *
 
 /**
  * g_i_marshalling_tests_int_out_out:
- * int0: (out):
- * int1: (out):
+ * @int0: (out):
+ * @int1: (out):
  */
 void
 g_i_marshalling_tests_int_out_out (gint *int0, gint *int1)
@@ -2848,7 +3397,7 @@ g_i_marshalling_tests_int_out_out (gint *int0, gint *int1)
 
 /**
  * g_i_marshalling_tests_int_return_out:
- * int_: (out):
+ * @int_: (out):
  */
 gint
 g_i_marshalling_tests_int_return_out (gint *int_)
