@@ -27,12 +27,12 @@
 #include <string.h>
 #include <unistd.h>
 #include "girffi.h"
-#include "girffi-private.h"
 #include "girepository.h"
+#include "girepository-private.h"
 
 ffi_type *
-g_ir_ffi_get_ffi_type (GITypeTag   tag,
-                       gboolean    is_pointer)
+_gi_type_tag_get_ffi_type (GITypeTag   tag,
+			   gboolean    is_pointer)
 {
   switch (tag)
     {
@@ -114,7 +114,7 @@ g_ir_ffi_get_ffi_type (GITypeTag   tag,
   g_assert_not_reached ();
 
   return NULL;
-}  
+}
 
 /**
  * g_type_info_get_ffi_type:
@@ -125,7 +125,7 @@ g_ir_ffi_get_ffi_type (GITypeTag   tag,
 ffi_type *
 g_type_info_get_ffi_type (GITypeInfo *info)
 {
-  return g_ir_ffi_get_ffi_type (g_type_info_get_tag (info), g_type_info_is_pointer (info));
+  return _gi_type_tag_get_ffi_type (g_type_info_get_tag (info), g_type_info_is_pointer (info));
 }
 
 /**
@@ -140,13 +140,13 @@ g_callable_info_get_ffi_arg_types (GICallableInfo *callable_info)
 {
     ffi_type **arg_types;
     gint n_args, i;
-    
+
     g_return_val_if_fail (callable_info != NULL, NULL);
 
     n_args = g_callable_info_get_n_args (callable_info);
-    
+
     arg_types = (ffi_type **) g_new0 (ffi_type *, n_args + 1);
-    
+
     for (i = 0; i < n_args; ++i)
       {
         GIArgInfo *arg_info = g_callable_info_get_arg (callable_info, i);
@@ -181,7 +181,7 @@ g_callable_info_get_ffi_return_type (GICallableInfo *callable_info)
   type_tag = g_type_info_get_tag (return_type);
   return_ffi_type = g_type_info_get_ffi_type (return_type);
   g_base_info_unref((GIBaseInfo*)return_type);
-  
+
   return return_ffi_type;
 }
 
@@ -198,6 +198,8 @@ g_callable_info_get_ffi_return_type (GICallableInfo *callable_info)
  * A primary intent of this function is that a dynamic structure allocated
  * by a language binding could contain a #GIFunctionInvoker structure
  * inside the binding's function mapping.
+ *
+ * Returns: %TRUE on success, %FALSE otherwise with @error set.
  */
 gboolean
 g_function_info_prep_invoker (GIFunctionInfo       *info,
@@ -212,12 +214,12 @@ g_function_info_prep_invoker (GIFunctionInfo       *info,
   gboolean is_method;
   gboolean throws;
   gint n_args, n_invoke_args, i;
-  
+
   g_return_val_if_fail (info != NULL, FALSE);
   g_return_val_if_fail (invoker != NULL, FALSE);
-  
+
   symbol = g_function_info_get_symbol ((GIFunctionInfo*) info);
-  
+
   if (!g_typelib_symbol (g_base_info_get_typelib((GIBaseInfo *) info),
                          symbol, &(invoker->native_address)))
     {
@@ -249,7 +251,7 @@ g_function_info_prep_invoker (GIFunctionInfo       *info,
 
   /* TODO: avoid malloc here? */
   atypes = g_malloc0 (sizeof (ffi_type*) * n_invoke_args);
-  
+
   if (is_method)
     {
       atypes[0] = &ffi_type_pointer;
@@ -286,7 +288,7 @@ g_function_info_prep_invoker (GIFunctionInfo       *info,
 /**
  * g_function_info_invoker_destroy:
  * @invoker: A #GIFunctionInvoker
- * 
+ *
  * Release all resources allocated for the internals of @invoker; callers
  * are responsible for freeing any resources allocated for the structure
  * itself however.
@@ -297,6 +299,11 @@ g_function_invoker_destroy (GIFunctionInvoker    *invoker)
   g_free (invoker->cif.arg_types);
 }
 
+typedef struct {
+  ffi_closure ffi_closure;
+  gpointer writable_self;
+} GIClosureWrapper;
+
 /**
  * g_callable_info_prepare_closure:
  * @callable_info: a callable info from a typelib
@@ -306,11 +313,8 @@ g_function_invoker_destroy (GIFunctionInvoker    *invoker)
  *
  * Prepares a callback for ffi invocation.
  *
- * Note: this function requires the heap to be executable, which
- *       might not function properly on systems with SELinux enabled.
- *
  * Return value: the ffi_closure or NULL on error.
- * The return value should be freed by calling g_callable_info_prepare_closure().
+ * The return value should be freed by calling g_callable_info_free_closure().
  */
 ffi_closure *
 g_callable_info_prepare_closure (GICallableInfo       *callable_info,
@@ -318,21 +322,21 @@ g_callable_info_prepare_closure (GICallableInfo       *callable_info,
                                  GIFFIClosureCallback  callback,
                                  gpointer              user_data)
 {
-  ffi_closure *closure;
+  gpointer exec_ptr;
+  GIClosureWrapper *closure;
   ffi_status status;
-    
+
   g_return_val_if_fail (callable_info != NULL, FALSE);
   g_return_val_if_fail (cif != NULL, FALSE);
   g_return_val_if_fail (callback != NULL, FALSE);
-    
-  closure = mmap (NULL, sizeof (ffi_closure),
-                  PROT_EXEC | PROT_READ | PROT_WRITE,
-                  MAP_ANON | MAP_PRIVATE, -1, sysconf (_SC_PAGE_SIZE));
+
+  closure = ffi_closure_alloc (sizeof (GIClosureWrapper), &exec_ptr);
   if (!closure)
     {
-      g_warning("mmap failed: %s\n", strerror(errno));
+      g_warning ("could not allocate closure\n");
       return NULL;
     }
+  closure->writable_self = closure;
 
   status = ffi_prep_cif (cif, FFI_DEFAULT_ABI,
                          g_callable_info_get_n_args (callable_info),
@@ -340,27 +344,23 @@ g_callable_info_prepare_closure (GICallableInfo       *callable_info,
                          g_callable_info_get_ffi_arg_types (callable_info));
   if (status != FFI_OK)
     {
-      g_warning("ffi_prep_cif failed: %d\n", status);
-      munmap(closure, sizeof (closure));
+      g_warning ("ffi_prep_cif failed: %d\n", status);
+      ffi_closure_free (closure);
       return NULL;
     }
 
-  status = ffi_prep_closure (closure, cif, callback, user_data);
+  status = ffi_prep_closure_loc (&closure->ffi_closure, cif, callback, user_data, exec_ptr);
   if (status != FFI_OK)
     {
       g_warning ("ffi_prep_closure failed: %d\n", status);
-      munmap(closure, sizeof (closure));
+      ffi_closure_free (closure);
       return NULL;
     }
 
-  if (mprotect(closure, sizeof (closure), PROT_READ | PROT_EXEC) == -1)
-    {
-      g_warning ("ffi_prep_closure failed: %s\n", strerror(errno));
-      munmap(closure, sizeof (closure));
-      return NULL;
-    }
-  
-  return closure;
+  /* Return exec_ptr, which points to the same underlying memory as
+   * closure, but via an executable-non-writable mapping.
+   */
+  return exec_ptr;
 }
 
 /**
@@ -374,5 +374,7 @@ void
 g_callable_info_free_closure (GICallableInfo *callable_info,
                               ffi_closure    *closure)
 {
-  munmap(closure, sizeof (closure));
+  GIClosureWrapper *wrapper = (GIClosureWrapper *)closure;
+
+  ffi_closure_free (wrapper->writable_self);
 }
