@@ -573,7 +573,7 @@ add_attribute_size (gpointer key, gpointer value, gpointer data)
   *size_p += ALIGN_VALUE (strlen (value_str) + 1, 4);
 }
 
-/* returns the full size of the blob including variable-size parts */
+/* returns the full size of the blob including variable-size parts (including attributes) */
 static guint32
 g_ir_node_get_full_size_internal (GIrNode *parent,
 				  GIrNode *node)
@@ -878,6 +878,8 @@ g_ir_node_get_full_size_internal (GIrNode *parent,
 	   node->name ? "' " : "",
 	   node, g_ir_node_type_to_string (node->type), size);
 
+  g_hash_table_foreach (node->attributes, add_attribute_size, &size);
+
   return size;
 }
 
@@ -887,14 +889,6 @@ g_ir_node_get_full_size (GIrNode *node)
   return g_ir_node_get_full_size_internal (NULL, node);
 }
 
-guint32
-g_ir_node_get_attribute_size (GIrNode *node)
-{
-  guint32 size = 0;
-  g_hash_table_foreach (node->attributes, add_attribute_size, &size);
-  return size;
-}
-
 int
 g_ir_node_cmp (GIrNode *node,
 		GIrNode *other)
@@ -1438,7 +1432,7 @@ g_ir_node_build_typelib (GIrNode         *node,
    */
   g_assert (node->offset == 0);
   node->offset = *offset;
-  build->offset_ordered_nodes = g_list_prepend (build->offset_ordered_nodes, node);
+  build->nodes_with_attributes = g_list_prepend (build->nodes_with_attributes, node);
 
   build->n_attributes += g_hash_table_size (node->attributes);
 
@@ -1648,6 +1642,8 @@ g_ir_node_build_typelib (GIrNode         *node,
 	blob->writable = prop->writable;
 	blob->construct = prop->construct;
 	blob->construct_only = prop->construct_only;
+	blob->transfer_ownership = prop->transfer;
+	blob->transfer_container_ownership = prop->shallow_transfer;
 	blob->reserved = 0;
 
         g_ir_node_build_typelib ((GIrNode *)prop->type,
@@ -1682,6 +1678,14 @@ g_ir_node_build_typelib (GIrNode         *node,
 	blob->symbol = write_string (function->symbol, strings, data, offset2);
 	blob->signature = signature;
 
+        /* function->result is special since it doesn't appear in the serialized format but
+         * we do want the attributes for it to appear
+         */
+        build->nodes_with_attributes = g_list_prepend (build->nodes_with_attributes, function->result);
+        build->n_attributes += g_hash_table_size (((GIrNode *) function->result)->attributes);
+        g_assert (((GIrNode *) function->result)->offset == 0);
+        ((GIrNode *) function->result)->offset = signature;
+
 	g_debug ("building function '%s'", function->symbol);
 
         g_ir_node_build_typelib ((GIrNode *)function->result->type,
@@ -1774,6 +1778,14 @@ g_ir_node_build_typelib (GIrNode         *node,
 	blob->name = write_string (node->name, strings, data, offset2);
 	blob->signature = signature;
 
+        /* signal->result is special since it doesn't appear in the serialized format but
+         * we do want the attributes for it to appear
+         */
+        build->nodes_with_attributes = g_list_prepend (build->nodes_with_attributes, signal->result);
+        build->n_attributes += g_hash_table_size (((GIrNode *) signal->result)->attributes);
+        g_assert (((GIrNode *) signal->result)->offset == 0);
+        ((GIrNode *) signal->result)->offset = signature;
+
         g_ir_node_build_typelib ((GIrNode *)signal->result->type,
 				 node, build, &signature, offset2);
 
