@@ -21,11 +21,10 @@
 # AnnotationParser - parses gtk-doc annotations
 
 import re
-import sys
 
 from .ast import (Array, Bitfield, Callback, Class, Enum, Field, Function,
-                  Interface, List, Map, Parameter, Record, Return, Type, Union,
-                  Varargs,
+                  Interface, List, Map, Parameter, Property, Record, Return,
+                  Type, Union, Varargs,
                   default_array_types,
                   BASIC_GIR_TYPES,
                   PARAM_DIRECTION_INOUT,
@@ -49,9 +48,10 @@ TAG_VFUNC = 'virtual'
 TAG_SINCE = 'since'
 TAG_DEPRECATED = 'deprecated'
 TAG_RETURNS = 'returns'
-TAG_RETURNS_ALT = 'return value'
 TAG_ATTRIBUTES = 'attributes'
 TAG_RENAME_TO = 'rename to'
+TAG_TYPE = 'type'
+TAG_TRANSFER = 'transfer'
 
 # Options - annotations for parameters and return values
 OPT_ALLOW_NONE = 'allow-none'
@@ -67,6 +67,7 @@ OPT_TYPE = 'type'
 OPT_CLOSURE = 'closure'
 OPT_DESTROY = 'destroy'
 OPT_SKIP = 'skip'
+OPT_FOREIGN = 'foreign'
 
 # Specific option values
 OPT_VAL_BITFIELD = 'bitfield'
@@ -97,14 +98,7 @@ class DocBlock(object):
         return '<DocBlock %r %r>' % (self.name, self.options)
 
     def get(self, name):
-        if name == TAG_RETURNS:
-            value = self.tags.get(name)
-            if value is None:
-                return self.tags.get(TAG_RETURNS_ALT)
-            else:
-                return value
-        else:
-            return self.tags.get(name)
+        return self.tags.get(name)
 
 
 class DocTag(object):
@@ -113,6 +107,7 @@ class DocTag(object):
         self.name = name
         self.options = {}
         self.comment = None
+        self.value = ''
 
     def __repr__(self):
         return '<DocTag %r %r>' % (self.name, self.options)
@@ -149,6 +144,10 @@ class Option(object):
 
 
 class AnnotationParser(object):
+    WHITESPACE_RE = re.compile(r'^\s*$')
+    ASCII_TEXT_RE = re.compile(r'\s*[A-Za-z]+')
+    OPTION_RE = re.compile(r'\([A-Za-z]+[^(]*\)')
+    RETURNS_RE = re.compile(r'^return(s?)( value)?:', re.IGNORECASE)
 
     def __init__(self, namespace, source_scanner, transformer):
         self._blocks = {}
@@ -192,56 +191,97 @@ class AnnotationParser(object):
         cpos = block_header.find(': ')
         if cpos:
             block_name = block_header[:cpos]
-            block_options, rest = self._parse_options(block_header[cpos+2:])
-            if rest:
-                return
+            block_options = self.parse_options(block_header[cpos+2:])
         else:
             block_name, block_options = block_header, {}
         block = DocBlock(block_name, block_options)
+        debug = block_name == 'annotation_object_compute_sum_n'
         comment_lines = []
+        parsing_parameters = True
+        last_param_tag = None
+
+        # Second phase: parse parameters, return values, Tag: format
+        # annotations.
+        #
+        # Valid lines look like:
+        # * @foo: some comment here
+        # * @baz: (inout): This has an annotation
+        # * @bar: (out) (allow-none): this is a long parameter comment
+        # *  that gets wrapped to the next line.
+        # *
+        # * Some documentation for the function.
+        # *
+        # * Returns: (transfer none): A value
         for line in comment[pos+1:].split('\n'):
             line = line.lstrip()
-            line = line[2:].strip() # Skip ' *'
-            if not line:
+            if (not line.startswith('*') or
+                self.WHITESPACE_RE.match(line[1:])):
+                # As soon as we find a line that's just whitespace,
+                # we're done parsing the parameters.
+                parsing_parameters = False
                 continue
-            if line.startswith('@'):
-                line = line[1:]
-            elif not ': ' in line:
+
+            line = line[1:].lstrip()
+
+            # Look for a parameter or return value.  Both of these can
+            # have parenthesized options.
+            first_colonspace_index = line.find(': ')
+            is_parameter = line.startswith('@')
+            is_return_value = self.RETURNS_RE.search(line)
+            if ((is_parameter or is_return_value)
+                and first_colonspace_index > 0):
+                if is_parameter:
+                    argname = line[1:first_colonspace_index]
+                else:
+                    argname = TAG_RETURNS
+                tag = DocTag(argname)
+                second_colon_index = line.rfind(':')
+                found_options = False
+                if second_colon_index > first_colonspace_index:
+                    value_line = \
+                      line[first_colonspace_index+2:second_colon_index]
+                    if self.OPTION_RE.search(value_line):
+                        # The OPTION_RE is a little bit heuristic.  If
+                        # we found two colons, we scan inside for something
+                        # that looks like (foo).
+                        # *Ideally* we'd change the gtk-doc format to
+                        # require double colons, and then there'd be
+                        # no ambiguity.  I.e.:
+                        # @foo:: Some documentation here
+                        # But that'd be a rather incompatible change.
+                        found_options = True
+                        tag.comment = line[second_colon_index+1:].strip()
+                        tag.options = self.parse_options(value_line)
+                if not found_options:
+                    # We didn't find any options, so just take the whole thing
+                    # as documentation.
+                    tag.comment = line[first_colonspace_index+2:].strip()
+                block.tags[argname] = tag
+                last_param_tag = tag
+            elif (not is_parameter) and parsing_parameters and last_param_tag:
+                # We need to handle continuation lines on parameters.  The
+                # conditional above - if a line doesn't start with '@', we're
+                # not yet in the documentation block for the whole function,
+                # and we've seen at least one parameter.
+                last_param_tag.comment += (' ' + line.strip())
+            elif first_colonspace_index > 0:
+                # The line is of the form "Tag: some value here", like:
+                # Since: 0.8
+                tag_name = line[:first_colonspace_index]
+                if self.ASCII_TEXT_RE.match(tag_name):
+                    tag_name = tag_name.lower()
+                    tag = DocTag(tag_name)
+                    tag.value = line[first_colonspace_index+2:]
+                    block.tags[tag_name] = tag
+                else:
+                    comment_lines.append(line)
+            elif (not is_parameter):
                 comment_lines.append(line)
-                continue
-            tag_name, value = self._split_tag_namevalue(line)
-            canon_name = tag_name.lower()
-            if canon_name in block.tags:
-                print >> sys.stderr, (
-                    "Symbol %s has multiple definition of tag %r" % (
-                    block_name, canon_name, ))
-            block.tags[canon_name] = self._create_tag(canon_name, value)
         block.comment = '\n'.join(comment_lines)
         self._blocks[block.name] = block
 
-    def _split_tag_namevalue(self, raw):
-        """Split a line into tag name and value"""
-        parts = raw.split(': ', 1)
-        if len(parts) == 1:
-            tag_name = parts[0]
-            value = ''
-            if tag_name.endswith(':'):
-                tag_name = tag_name[:-1]
-        else:
-            tag_name, value = parts
-        return (tag_name, value)
-
-    def _create_tag(self, tag_name, value):
-        # Tag: bar
-        # Tag: bar opt1 opt2
-        tag = DocTag(tag_name)
-        tag.value = value
-        options, rest = self._parse_options(tag.value)
-        tag.options = options
-        tag.comment = rest
-        return tag
-
-    def _parse_options(self, value):
+    @classmethod
+    def parse_options(cls, value):
         # (foo)
         # (bar opt1 opt2...)
         opened = -1
@@ -266,11 +306,7 @@ class AnnotationParser(object):
                 last = i + 2
                 opened = -1
 
-        if last is not None:
-            rest = value[last:].strip()
-        else:
-            rest = None
-        return options, rest
+        return options
 
 
 class AnnotationApplier(object):
@@ -409,6 +445,17 @@ class AnnotationApplier(object):
         if block:
             prop.doc = block.comment
 
+        transfer_tag = self._get_tag(block, TAG_TRANSFER)
+        if transfer_tag is not None:
+            options = {OPT_TRANSFER: Option(transfer_tag.value)}
+        else:
+            options = {}
+        prop.transfer = self._extract_transfer(parent, prop, options)
+
+        type_tag = self._get_tag(block, TAG_TYPE)
+        if type_tag:
+            prop.type = self._resolve(type_tag.value, prop.type)
+
     def _parse_callback(self, callback):
         block = self._blocks.get(callback.ctype)
         self._parse_node_common(callback, block)
@@ -475,7 +522,7 @@ class AnnotationApplier(object):
             for vfunc in parent.virtual_methods:
                 if vfunc.name == invoker_name:
                     matched = True
-                    vfunc.invoker = meth.name
+                    vfunc.invoker = meth
                     break
             if not matched:
                 print "warning: unmatched virtual invoker %r for method %r" % \
@@ -484,6 +531,11 @@ class AnnotationApplier(object):
     def _parse_vfunc(self, parent, vfunc):
         key = '%s::%s' % (parent.type_name, vfunc.name)
         self._parse_callable(vfunc, self._blocks.get(key))
+        if vfunc.invoker:
+            # We normally expect annotations like (element-type) to be
+            # applied to the invoker.
+            block = self._blocks.get(vfunc.invoker.symbol)
+            self._parse_callable(vfunc, block)
 
     def _parse_field(self, parent, field, block=None):
         if isinstance(field, Callback):
@@ -586,13 +638,15 @@ class AnnotationApplier(object):
 
     def _parse_param_ret_common(self, parent, node, tag):
         options = getattr(tag, 'options', {})
-        node.direction = self._extract_direction(node, options)
+        (node.direction, node.caller_allocates) = \
+            self._extract_direction(node, options)
         container_type = self._extract_container_type(
             parent, node, options)
         if container_type is not None:
             node.type = container_type
         if node.direction is None:
             node.direction = self._guess_direction(node)
+            node.caller_allocates = False
         node.transfer = self._extract_transfer(parent, node, options)
         param_type = options.get(OPT_TYPE)
         if param_type:
@@ -606,17 +660,39 @@ class AnnotationApplier(object):
         if tag is not None and tag.comment is not None:
             node.doc = tag.comment
 
+        for key in options:
+            if '.' in key:
+                value = options.get(key)
+                if value:
+                    node.attributes.append((key, value.one()))
+
     def _extract_direction(self, node, options):
+        caller_allocates = False
         if (OPT_INOUT in options or
             OPT_INOUT_ALT in options):
             direction = PARAM_DIRECTION_INOUT
         elif OPT_OUT in options:
+            subtype = options[OPT_OUT]
+            if subtype is not None:
+                subtype = subtype.one()
             direction = PARAM_DIRECTION_OUT
+            if subtype in (None, ''):
+                if (node.type.name not in BASIC_GIR_TYPES) and node.type.ctype:
+                    caller_allocates = '**' not in node.type.ctype
+                else:
+                    caller_allocates = False
+            elif subtype == 'caller-allocates':
+                caller_allocates = True
+            elif subtype == 'callee-allocates':
+                caller_allocates = False
+            else:
+                raise InvalidAnnotationError(
+                    "out direction for %s is invalid (%r)" % (node, subtype))
         elif OPT_IN in options:
             direction = PARAM_DIRECTION_IN
         else:
             direction = node.direction
-        return direction
+        return (direction, caller_allocates)
 
     def _guess_array(self, node):
         ctype = node.type.ctype
@@ -628,15 +704,20 @@ class AnnotationApplier(object):
             return True
         return False
 
+    def _is_array_type(self, node):
+        if node.type.name in ['GLib.Array', 'GLib.PtrArray',
+                              'GLib.ByteArray']:
+            return True
+        if node.type.ctype in ['GArray*', 'GPtrArray*', 'GByteArray*']:
+            return True
+        return False
+
     def _extract_container_type(self, parent, node, options):
         has_element_type = OPT_ELEMENT_TYPE in options
         has_array = OPT_ARRAY in options
 
         if not has_array:
-            has_array = \
-                node.type.name in ['GLib.Array', 'GLib.PtrArray',
-                                   'GLib.ByteArray'] or \
-                node.type.ctype in ['GArray*', 'GPtrArray*', 'GByteArray*']
+            has_array = self._is_array_type(node)
 
         # FIXME: This is a hack :-(
         if (not isinstance(node, Field) and
@@ -663,9 +744,7 @@ class AnnotationApplier(object):
         else:
             array_values = {}
 
-        is_g_array = node.type.ctype.startswith('GArray*') or \
-                     node.type.ctype.startswith('GPtrArray*') or \
-                     node.type.ctype.startswith('GByteArray*')
+        is_g_array = self._is_array_type(node)
 
         element_type = options.get(OPT_ELEMENT_TYPE)
         if element_type is not None:
@@ -762,6 +841,10 @@ class AnnotationApplier(object):
                 node.type.ctype,
                 self._resolve(element_type[0]),
                 self._resolve(element_type[1]))
+        elif self._is_array_type(node):
+            container_type = Array(node.type.name,
+                                   node.type.ctype,
+                                   self._resolve(element_type[0]))
         else:
             print 'FIXME: unhandled element-type container:', node
         return container_type
@@ -787,6 +870,7 @@ class AnnotationApplier(object):
         self._parse_deprecated(node, block)
         self._parse_attributes(node, block)
         self._parse_skip(node, block)
+        self._parse_foreign(node, block)
 
     def _parse_version(self, node, block):
         since_tag = self._get_tag(block, TAG_SINCE)
@@ -812,7 +896,8 @@ class AnnotationApplier(object):
         annos_tag = self._get_tag(block, TAG_ATTRIBUTES)
         if annos_tag is None:
             return
-        for key, value in annos_tag.options.iteritems():
+        options = AnnotationParser.parse_options(annos_tag.value)
+        for key, value in options.iteritems():
             if value:
                 node.attributes.append((key, value.one()))
 
@@ -821,6 +906,11 @@ class AnnotationApplier(object):
             if OPT_SKIP in block.options:
                 node.skip = True
 
+    def _parse_foreign(self, node, block):
+        if block is not None:
+            if OPT_FOREIGN in block.options:
+                node.foreign = True
+
     def _parse_rename_to_func(self, node, block):
         rename_to_tag = self._get_tag(block, TAG_RENAME_TO)
         if rename_to_tag is None:
@@ -878,6 +968,8 @@ class AnnotationApplier(object):
         elif isinstance(node, Parameter):
             if node.direction in [PARAM_DIRECTION_INOUT,
                                   PARAM_DIRECTION_OUT]:
+                if node.caller_allocates:
+                    return PARAM_TRANSFER_NONE
                 return PARAM_TRANSFER_FULL
             # This one is a hack for compatibility; the transfer
             # for string parameters really has no defined meaning.
@@ -886,7 +978,11 @@ class AnnotationApplier(object):
             else:
                 return PARAM_TRANSFER_NONE
         elif isinstance(node, Return):
-            if (node.type.canonical in BASIC_GIR_TYPES or
+            if (isinstance(node.type, Array) and
+                    node.type.element_type is not None and
+                    node.type.element_type.name == 'utf8'):
+                return PARAM_TRANSFER_FULL
+            elif (node.type.canonical in BASIC_GIR_TYPES or
                 (node.type.canonical in [TYPE_NONE, TYPE_ANY] and
                  node.type.is_const)):
                 return PARAM_TRANSFER_NONE
@@ -894,5 +990,7 @@ class AnnotationApplier(object):
                 return PARAM_TRANSFER_FULL
         elif isinstance(node, Field):
             return PARAM_TRANSFER_NONE
+        elif isinstance(node, Property):
+            return PARAM_TRANSFER_NONE
         else:
             raise AssertionError(node)
