@@ -173,12 +173,14 @@ class Node(object):
 
     def __init__(self, name=None):
         self.name = name
-        self.attributes = [] # (key, value)*
         self.skip = False
+        self.introspectable = True
+        self.attributes = [] # (key, value)*
         self.deprecated = None
         self.deprecated_version = None
         self.version = None
         self.foreign = False
+        self.file_positions = set()
 
     def __cmp__(self, other):
         return cmp(self.name, other.name)
@@ -189,6 +191,16 @@ class Node(object):
     def remove_matching_children(self, pred):
         pass
 
+    def inherit_file_positions(self, node):
+        self.file_positions.update(node.file_positions)
+
+    def add_file_position(self, filename, line, column):
+        self.file_positions.add((filename, line, column))
+
+    def add_symbol_reference(self, symbol):
+        if symbol.source_filename:
+            self.add_file_position(symbol.source_filename, symbol.line, -1)
+
 class Namespace(Node):
 
     def __init__(self, name, version):
@@ -521,12 +533,12 @@ class Constant(Node):
             self.name, self.type, self.value)
 
 
-class Property(Node):
+class Property(TypeContainer):
 
     def __init__(self, name, type_name, readable, writable,
-                 construct, construct_only, ctype=None):
-        Node.__init__(self, name)
+                 construct, construct_only, ctype=None, transfer=None):
         self.type = Type(type_name, ctype)
+        TypeContainer.__init__(self, name, self.type, transfer)
         self.readable = readable
         self.writable = writable
         self.construct = construct
