@@ -58,7 +58,7 @@ def _get_option_parser():
                       help="full path to libtool")
     parser.add_option("", "--no-libtool",
                       action="store_true", dest="nolibtool", default=False,
-                      help="use libtool")
+                      help="do not use libtool")
     parser.add_option("-l", "--library",
                       action="append", dest="libraries", default=[],
                       help="libraries of this unit")
@@ -75,6 +75,9 @@ def _get_option_parser():
     parser.add_option("", "--strip-prefix",
                       action="store", dest="strip_prefix", default=None,
                       help="remove this prefix from objects and functions")
+    parser.add_option("", "--add-init-section",
+                      action="append", dest="init_sections", default=[],
+            help="add extra initialization code in the introspection program")
     parser.add_option("-o", "--output",
                       action="store", dest="output",
                       help="output to writeout, defaults to stdout")
@@ -84,6 +87,12 @@ def _get_option_parser():
     parser.add_option("", "--pkg-export",
                       action="append", dest="packages_export", default=[],
                       help="Associated pkg-config packages for this library")
+    parser.add_option('', "--warn-all",
+                      action="store_true", dest="warn_all", default=False,
+                      help="If true, enable all warnings for introspection")
+    parser.add_option('', "--warn-error",
+                      action="store_true", dest="warn_fatal",
+                      help="Turn warnings into fatal errors")
     parser.add_option("-v", "--verbose",
                       action="store_true", dest="verbose",
                       help="be verbose")
@@ -268,12 +277,18 @@ def scanner_main(args):
         transformer.set_strip_prefix(options.strip_prefix)
     else:
         transformer.set_strip_prefix(options.namespace_name)
+    if options.warn_all:
+        transformer.enable_warnings(True)
     transformer.set_include_paths(options.include_paths)
     shown_include_warning = False
     for include in options.includes:
         if os.sep in include:
             raise ValueError("Invalid include path %r" % (include, ))
-        include_obj = Include.from_string(include)
+        try:
+            include_obj = Include.from_string(include)
+        except:
+            sys.stderr.write("Malformed include %r\n" % (include, ))
+            sys.exit(1)
         transformer.register_include(include_obj)
 
     packages = set(options.packages)
@@ -321,6 +336,11 @@ def scanner_main(args):
     except InvalidAnnotationError, e:
         raise SystemExit("ERROR in annotation: %s" % (str(e), ))
 
+    glibtransformer.final_analyze()
+
+    if options.warn_fatal and transformer.did_warn():
+        return 1
+
     # Write out AST
     if options.packages_export:
         exported_packages = options.packages_export
