@@ -19,6 +19,7 @@
 #
 
 import os
+import sys
 
 from .ast import (Bitfield, Callback, Enum, Function, Namespace, Member,
                   Parameter, Return, Struct, Field,
@@ -64,6 +65,7 @@ class Names(object):
 class Transformer(object):
 
     def __init__(self, cachestore, namespace_name, namespace_version):
+        self._cwd = os.getcwd() + os.sep
         self._cachestore = cachestore
         self.generator = None
         self._namespace = Namespace(namespace_name, namespace_version)
@@ -71,6 +73,8 @@ class Transformer(object):
         self._pkg_config_packages = set()
         self._typedefs_ns = {}
         self._strip_prefix = ''
+        self._enable_warnings = False
+        self._warned = False
         self._includes = set()
         self._includepaths = []
 
@@ -86,6 +90,12 @@ class Transformer(object):
     def get_strip_prefix(self):
         return self._strip_prefix
 
+    def enable_warnings(self, enable):
+        self._enable_warnings = enable
+
+    def did_warn(self):
+        return self._warned
+
     def get_pkgconfig_packages(self):
         return self._pkg_config_packages
 
@@ -114,6 +124,58 @@ class Transformer(object):
 
     # Private
 
+    def log_warning(self, text, file_positions=None, prefix=None):
+        """Log a warning, using optional file positioning information.
+If the warning is related to a Node type, see log_node_warning()."""
+        if not self._enable_warnings:
+            return
+
+        if file_positions is None or len(file_positions) == 0:
+            target_file_positions = [('<unknown>', -1, -1)]
+        else:
+            target_file_positions = file_positions
+
+        for (filename, line, column) in target_file_positions:
+            if filename.startswith(self._cwd):
+                filename = filename[len(self._cwd):]
+            if column != -1:
+                position = '%s:%d:%d' % (filename, line, column)
+            elif line != -1:
+                position = '%s:%d' % (filename, line, )
+            else:
+                position = '%s:' % (filename, )
+
+        if prefix:
+            print >>sys.stderr, \
+'''%s: warning: ns=%r %s: %s''' % (position, self._namespace.name,
+                                   prefix, text)
+        else:
+            print >>sys.stderr, \
+'''%s: warning: ns=%r: %s''' % (position, self._namespace.name, text)
+
+    def log_symbol_warning(self, symbol, text):
+        """Log a warning in the context of the given symbol."""
+        file_positions = [(symbol.source_filename, symbol.line, -1)]
+        prefix = "symbol=%r" % (symbol.ident, )
+        self.log_warning(text, file_positions, prefix=prefix)
+
+    def log_node_warning(self, node, text, context=None):
+        """Log a warning, using information about file positions from
+the given node.  The optional context argument, if given, should be
+another Node type which will also be displayed.  If no file position
+information is available from the node, the position data from the
+context will be used."""
+        if len(node.file_positions) == 0 and \
+                (context is not None) and len(context.file_positions) > 0:
+            file_positions = context.file_positions
+        else:
+            file_positions = node.file_positions
+
+        if context:
+            text = "context=%r %s" % (context.name, text)
+
+        self.log_warning(text, file_positions)
+
     def _find_include(self, include):
         searchdirs = self._includepaths[:]
         for path in _xdg_data_dirs:
@@ -125,8 +187,9 @@ class Transformer(object):
             path = os.path.join(d, girname)
             if os.path.exists(path):
                 return path
-        raise ValueError("Couldn't find include %r (search path: %r)"\
+        sys.stderr.write("Couldn't find include %r (search path: %r)\n"\
                          % (girname, searchdirs))
+        sys.exit(1)
 
     def _parse_include(self, filename):
         parser = self._cachestore.load(filename)
@@ -214,15 +277,13 @@ class Transformer(object):
 
     def _enum_common_prefix(self, symbol):
         def common_prefix(a, b):
-            alen = len(a)
-            blen = len(b)
-            l = min(alen, blen)
-            for i in xrange(l):
-                if a[i] != b[i]:
-                    return a[:i]
-            if alen > blen:
-                return b
-            return a
+            commonparts = []
+            for aword, bword in zip(a.split('_'), b.split('_')):
+                if aword != bword:
+                    return '_'.join(commonparts) + '_'
+                commonparts.append(aword)
+            return min(a, b)
+
         # Nothing less than 2 has a common prefix
         if len(list(symbol.base_type.child_list)) < 2:
             return None
@@ -261,16 +322,21 @@ class Transformer(object):
         else:
             klass = Enum
         node = klass(enum_name, symbol.ident, members)
+        node.add_symbol_reference(symbol)
         self._names.type_names[symbol.ident] = (None, node)
         return node
 
     def _create_object(self, symbol):
-        return Member(symbol.ident, symbol.base_type.name,
+        node = Member(symbol.ident, symbol.base_type.name,
                       symbol.ident)
+        node.add_symbol_reference(symbol)
+        return node
 
     def _type_is_callback(self, type):
-        if (isinstance(type, Callback) or
-            isinstance(self._typedefs_ns.get(type.name), Callback)):
+        if isinstance(type, Callback):
+            return True
+        node = self._names.names.get(type.name)
+        if node and isinstance(node[1], Callback):
             return True
         return False
 
@@ -283,9 +349,8 @@ class Transformer(object):
         return False
 
     def _handle_destroy(self, param, destroy_idx, destroy_param):
-        if ((self._namespace.name == 'GLib' and
-             destroy_param.type.name == 'DestroyNotify') or
-            destroy_param.type.name == 'GLib.DestroyNotify'):
+        if (destroy_param.type.name == 'GLib.DestroyNotify' or
+            destroy_param.type.ctype == 'GDestroyNotify'):
             param.destroy_name = destroy_param.name
             param.destroy_index = destroy_idx
             return True
@@ -296,6 +361,10 @@ class Transformer(object):
             if not self._type_is_callback(param.type):
                 continue
 
+            # set a default scope
+            if param.scope is None:
+                param.scope = 'call'
+
             # j is the index where we look for closure/destroy to
             # group with the callback param
             j = i + 1
@@ -322,6 +391,7 @@ class Transformer(object):
         self._augment_callback_params(parameters)
         name = self._strip_namespace_func(symbol.ident)
         func = Function(name, return_, parameters, symbol.ident)
+        func.add_symbol_reference(symbol)
         return func
 
     def _create_source_type(self, source_type):
@@ -368,7 +438,7 @@ class Transformer(object):
                 else:
                     derefed_name = canonical_ctype
                 derefed_name = self.resolve_param_type(derefed_name)
-                ftype = Array(ctype, self.parse_ctype(derefed_name))
+                ftype = Array(None, ctype, self.parse_ctype(derefed_name))
                 child_list = list(symbol.base_type.child_list)
                 ftype.zeroterminated = False
                 if child_list:
@@ -381,6 +451,7 @@ class Transformer(object):
             # (except for Objects, see also glibtransformer.py)
             node = Field(symbol.ident, ftype, ftype.name,
                          readable=True, writable=True, bits=symbol.const_int)
+            node.add_symbol_reference(symbol)
         return node
 
     def _create_typedef(self, symbol):
@@ -504,7 +575,7 @@ class Transformer(object):
         if (symbol.source_filename is None or
             not symbol.source_filename.endswith('.h')):
             return None
-        name = self.remove_prefix(symbol.ident)
+        name = self._strip_namespace_func(symbol.ident)
         if symbol.const_string is not None:
             type_name = 'utf8'
             value = symbol.const_string
@@ -518,11 +589,13 @@ class Transformer(object):
             raise AssertionError()
 
         const = Constant(name, type_name, value)
+        const.add_symbol_reference(symbol)
         return const
 
     def _create_typedef_struct(self, symbol, disguised=False):
         name = self.remove_prefix(symbol.ident)
         struct = Struct(name, symbol.ident, disguised)
+        struct.add_symbol_reference(symbol)
         self._typedefs_ns[symbol.ident] = struct
         self._create_struct(symbol)
         return struct
@@ -530,6 +603,7 @@ class Transformer(object):
     def _create_typedef_union(self, symbol):
         name = self.remove_prefix(symbol.ident)
         union = Union(name, symbol.ident)
+        union.add_symbol_reference(symbol)
         self._typedefs_ns[symbol.ident] = union
         self._create_union(symbol)
         return union
@@ -565,6 +639,7 @@ class Transformer(object):
             if field:
                 compound.fields.append(field)
 
+        compound.add_symbol_reference(symbol)
         return compound
 
     def _create_struct(self, symbol, anonymous=False):
@@ -588,6 +663,7 @@ class Transformer(object):
         else:
             name = self.remove_prefix(symbol.ident)
         callback = Callback(name, retval, parameters, symbol.ident)
+        callback.add_symbol_reference(symbol)
 
         return callback
 
@@ -667,7 +743,7 @@ class Transformer(object):
                                                      self.ctype_of(ptype),
                                                      names, **kwargs)
         elif isinstance(ptype, basestring):
-            return self.resolve_type_name_full(ptype, None, names, **kwargs)
+            return self.resolve_type_name_full(ptype, ptype, names, **kwargs)
         else:
             raise AssertionError("Unhandled param: %r" % (ptype, ))
         return ptype
@@ -687,3 +763,8 @@ class Transformer(object):
             else:
                 break
         return type_name
+
+    def iter_enums(self):
+        for node in self._namespace.nodes:
+            if isinstance(node, Enum):
+                yield node
